import re
import urlparse, urllib


# helpers

def d2s(dct, keys):
    "Returns values from dict as a sequence"
    return [dct[key] for key in keys]

def s2d(tup, keys):
    "Returns values from tuple as a  dict mapped to keys"
    dct = {}
    for i in range(len(keys)):
        dct[keys[i]] = tup[i]
    return dct


class LinkParser:

    "Handles parsing of links in urls"

    # typically: '<a href="/link">more</a>', '<a href="', 'href', '/link', '">more</a>'
    link_keys = ('tag', 'tagstart', 'attrname', 'url', 'tagend')

    def __init__(self, content, baseUrl):
        # find all links in text
        # the match returned has values in link_keys
        self.baseUrl = baseUrl
        self.setContent(content)


    def setContent(self, content):
        "Set the text/content of the page"
        self.content = content
        self._update_urls()


    def _update_urls(self):
        "Updates the urls. Nessecary if self.text is changed"
        #urlPattern = '''((<[^>]*?(href|src|background) *=[ "']*)(.*?)([ "'].*?>))'''
        #h4ck3d: added support for forms (action)
        urlPattern = '''((<[^>]*?(href|src|background) *=[ "']*)(.*?)([ "'].*?>))'''
        linkPattern = re.compile(urlPattern, re.I|re.S)
        content = self.getContent()
        self.urls = []
        for url in linkPattern.findall(content):
            url_match = s2d(url, self.link_keys)
            if url_match['url'][:2] == '//':
                url_match['url'] = 'http:' + url_match['url']
            self.urls.append(url_match)


    def getContent(self):
        "Set the text/content of the page"
        return self.content


    def relativeUrls(self):
        "Returns only the relative urls"
        result = []
        for match_ in self.urls:
            # some buggers leaves an empty url. Browsers handles this as '/'
            url = match_['url']
            if not url: url = '/'
            scheme, location, path, params, query, fragment = urlparse.urlparse(url)
            if not scheme and url[0] != '#' and url[:2] != '//':
                result.append(match_)
        return result


    def replacer(self, rDict):
        """
        Case sensitive multireplacer. Replaces keys with values from rDict. 
        Keys must be non-overlapping!.
        """
        content = self.getContent()
        # find all matches
        matches = [] # list of tuples (position, key)
        for key, val in rDict.items():
            cursor = content.find(key)
            while cursor != -1:
                matches.append((cursor, key))
                cursor = content.find(key, cursor + len(key))
        matches.sort() # need to start from beginning to work
        # do the actual replacing
        result = []
        cursor = 0
        for matchStart, key in matches:
            result.append(content[cursor:matchStart])
            result.append(rDict[key])
            cursor = matchStart + len(key)
        result.append(content[cursor:]) # also get the last bit
        self.setContent(''.join(result))


    def relative2abs(self):
        "returns a dict of {relative:absolute} key vals"
        baseUrl = self.baseUrl
        replaceDict = {}
        for r in self.relativeUrls():
            url = r['url']
            absUrl = urlparse.urljoin(baseUrl, url)
            replaceDict[r['tag']] = r['tagstart'] + urlparse.urljoin(baseUrl, url) + r['tagend']
        self.replacer(replaceDict)


    def anchors2proxies(self, url_path, proxy_url_name):
        "Converts all anchors to the from the baseurl to proxy urls"
        baseUrl = self.baseUrl
        baseLocation = urlparse.urlparse(baseUrl)[1]
        b2p_map = {} # base2proxy_mapping
        for url in self.urls:
            scheme, location, path, params, query, fragment = urlparse.urlparse(url['url'])
            # h4ck3d: Skip rewriting for .tar files
            #if ".tar" in path: continue
            if ".tar" in path: url_path = "gettar"
            # h4ck3d: Added support for forms (action)
            if location == baseLocation and url['attrname'].lower() in ['href', 'action']:
                #if ".tar" in path: url_path = "/wrapper/getfile"
                if fragment:
                    proxy_uri = "%s?%s=%s#%s" % (
                        url_path, proxy_url_name, urllib.quote_plus(baseUrl), fragment
                    )
                else:
                    proxy_uri = "%s?%s=%s" % (url_path, proxy_url_name, urllib.quote_plus(url['url']))
                b2p_map[url['tag']] = url['tagstart'] + proxy_uri + url['tagend']
        self.replacer(b2p_map)


    def asAbsolute(self, url_path, proxy_url_name):
        "Returns text with relative links as absolutes"
        self.relative2abs()
        self.anchors2proxies(url_path, proxy_url_name)
        return self.getContent()


if __name__ == '__main__':

#    dct = {'first':'Max M', 'last':'Rasmussen', 'age':'39'}
#    keys = ('first', 'last', 'age')
#    values = ('Max M','Rasmussen','42')
#    print dct
#    print d2s(dct, keys)
#    print s2d(values, keys)

    #f = file('c:/temp/wired.htm')
#    f = file('C:/temp/slashdot.htm')
    f = file('C:/temp/fyens.htm')
#    f = file('C:/temp/tietgen.htm')
#    f = file('C:/temp/eb.htm')
    text = f.read()
    f.close()

    lp = LinkParser(text, 'http://www.fyens.dk/')
    print lp.asAbsolute()
