# Zope
from OFS import SimpleItem
from OFS.PropertyManager import PropertyManager
import Globals # so I can get Globals data_dir
from Globals import HTMLFile
from AccessControl import getSecurityManager, ClassSecurityInfo

# python
import urllib, urllib2, re, os, os.path
from urlparse import urljoin


try:
    import ClientCookie
except:
    ClientCookie = None
    
try:
    import cookielib
except:
    cookielib = None

# product
import LinkParser
from BeautifulSoup import BeautifulSoup



class mxmProxyTool(PropertyManager, SimpleItem.SimpleItem):

    """
    Tool for browsing other sites via. a proxy.
    """

    meta_type = 'mxm ProxyTool'

    id = 'mxm_proxytool'

    # this is the default key for the remote url
    MXM_PROXY_URL_NAME = 'mxm_proxy_url'

    # Standard security settings
    security = ClassSecurityInfo()

    manage_options = PropertyManager.manage_options + \
                     SimpleItem.SimpleItem.manage_options

    _properties=(
        {'id':'title', 'type':'string', 'mode':'w'},
        {'id':'MXM_PROXY_URL_NAME', 'type':'string', 'mode':'w'},
    )

    def __init__(self):
        self.title = self.meta_type

    def _getUserId(self):
        "Returns the userid"
        "Returns_userId"
        name = ''
        user = getSecurityManager().getUser()
        if user is not None:
            name=user.getUserName()
        return name

    # Saving & loading cookies per authenticated user

    def _getCookieFileName(self):
        "Returns the path to the current users cookiefile"
        cookiejar_name = os.sep.join(
            ('', 'mxm-proxy-cookies', '%s-cookiejar.txt' % self._getUserId())
        )
        filename = str(Globals.data_dir) + cookiejar_name
        return filename

    def _getCookieJar(self):
        "Returns the current users cookiejar, or a new one"
        cookieJar = ClientCookie.CookieJar()
        cookieJar.filename = self._getCookieFileName()
        if os.path.exists(cookieJar.filename):
            cookieJar.load(ignore_discard=True)
        return cookieJar

    def _setCookieJar(self, cookieJar):
        "Returns the current users cookiejar, or a new one"
        cookieJar.save(ignore_discard=True)

    # public methods

    def browse(self, proxy_url='', allowed_urls='', data=None, body_regex="", bodyonly=1):
        """
        Returns the content of a webpage and rewrite links
        proxy_url: the page content should be fetched from
        allowed_urls: A regex string showing allowed url patterns
        data: a data dict like defined in pythons urlopen()
        bodyonly: if true it onlye shows content from inside the <body></body> 
        tags
        """
        #return "%s %s %s %s" % (proxy_url, allowed_urls, data, body_regex)
        req = self.REQUEST
        content = ''
        try:
	    #raise Exception("HERE: '%s'" % proxy_url)
            if proxy_url == '':
                proxy_url = req.get(self.MXM_PROXY_URL_NAME, '')
            # is the proxy_url allowed?
            if allowed_urls:
                urlfilter = re.compile(allowed_urls)
                if not urlfilter.search(proxy_url):
                    proxy_url = ''
            if proxy_url:
                content = self.urlopen(proxy_url, data)
                if bodyonly:
                    content = self.filterBody(content)
		    
		# --- h4ck3d: Cut content following body_regex    
		
                if not body_regex == "":
                    bodyPattern = re.compile(body_regex, re.I|re.S)
                    bodyContent = bodyPattern.search(content)
		    if not bodyContent is None:
		        content = bodyContent.group(0)
		
		# --- /h4ck3d	
		
                lp = LinkParser.LinkParser(content, proxy_url)
                content = lp.asAbsolute(req.URL, self.MXM_PROXY_URL_NAME)
        except Exception, ex:
            content = "Exception caught: '%s' (URL is %s)" % (ex, proxy_url)

        # Add delimiters
        content = "<!-- wrapper start -->\n%s\n<!-- wrapper end -->" % content
        
        return content

    def urlopen(self, url, data=None):
        """
        opens a page and returns the content. Can also be used for logging in to 
        a site with cookie based authentication.
        
        This is the actual workhorse ot the tool. It also saves and 
        loads cookies for logged in users, so they can browse sites that 
        requires a login, or other cookies.
        """
	

	# --- h4ck3d: Send Plone login cookie to remote server
	
	if data == "PLONE_LOGIN":
	    # Get Plone login cookie and write cookie Header
	    cookie_header = { 'Cookie' : self.REQUEST.get('HTTP_COOKIE', '') }
            cookie_header['Referer'] = (self.site)
	    data = ""
	else:
	    cookie_header = {} 
	
	# --- /h4ck3d

	
        if ClientCookie:
            cookies = self._getCookieJar()
            opener = ClientCookie.build_opener(ClientCookie.HTTPCookieProcessor(cookies))
            request = ClientCookie.Request(url, data, cookie_header)
            response = opener.open(request, data)
            content = response.read()
            response.close()
            self._setCookieJar(cookies)
        else:
            request = urllib2.Request(url, data, cookie_header)
            response = urllib2.urlopen(request)
            content = response.read()
            response.close()
        return content

    def getSoup(self, content):
        """
        Parses html content with BeautifulSoup. Nice for external methods Check 
        the web for documentation.
        http://www.crummy.com/software/BeautifulSoup/
        """
        soup = BeautifulSoup()
        soup.feed(content)
        return soup

    def filterBody(self, source):
        "Returns content between html body tags"
        urlPattern = '''((<body.*?>(.*?)</body>))'''
        linkPattern = re.compile(urlPattern, re.I|re.S)
        result = linkPattern.search(source)
        if result is None:
            return ''
        return result.group(0)

    #####################################
    # h4ck3d: Site selector by M. Pinotti
    def showSite(self, site):
        req = self.REQUEST
        p = self.MXM_PROXY_URL_NAME
	debug = ""

	# Read config file
	from configobj import ConfigObj
        cfile = os.path.join(INSTANCE_HOME, 'Products', 'mxmProxyTool', 'mxmProxyTool.conf')
        config = ConfigObj(cfile)

	# Select site
	try: config = config[site]
        except KeyError:
          return "KeyError: '%s' not found in config file (%s)" % (site, cfile)
	
        # Save site (used to sendo correct referer)
        self.site = site
        
	# Parse config
	url = req.get(p, config['url'])
	allowed = config['allowed']
	regex = config['regex']
	data = config['data']
	
	# Build data
	if not config['data'] == 'PLONE_LOGIN':
	    data = ""
	    for param in config['data']:
	        data += param+"=%s&" % req.get(param, "")

	# Build query string
	# These are parameters used by mxmProxyTool
	forbidden_params = [p, 'mxm_proxy_path', 'portal_status_message']
	query_string = ""
	# proxy_path is the path to add to the http request
	proxy_path = ""
	for param in req.get('QUERY_STRING').split('&'):
	    kvpair = param.split('=')
	    if not len(kvpair) == 2: continue
	    if kvpair[0] == 'mxm_proxy_path': proxy_path = kvpair[1]
	    elif not kvpair[0] in forbidden_params: query_string += param+"&"

	# Add query string 
	if not proxy_path == "": url += '/'+urllib.unquote(proxy_path)
	if not query_string == "": url += "?"+query_string
	
        # Get site content
	debug += "URL: %s <br />" % url
	debug = "<blockquote>DEBUG: %s</blockquote>" % debug 
        return unicode(self.browse(url, allowed, data, regex), "latin-1")



##########################
# constructor pages. Only used when the product is added to a folder.

def manage_addmxmProxyTool(self, REQUEST=None):
    "Add the proxytool to the current objectmanager."
    self._setObject(mxmProxyTool.id, mxmProxyTool())
    # create cookiejar directory for saving cookiejars
    cookie_dir = str(Globals.data_dir) + os.sep + 'mxm-proxy-cookies'
    # h4ck3d: avoid error caused by a double mxmProxyTool instance
    try: os.mkdir(cookie_dir)
    except: pass
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

# Get user data from this form
manage_addmxmProxyToolForm = HTMLFile('manage_addmxmProxyToolForm', globals())

